var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target, mod));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/earthmap.ts
var earthmap_exports = {};
__export(earthmap_exports, {
  EarthMap: () => EarthMap
});
module.exports = __toCommonJS(earthmap_exports);
var import_l7_utils = require("@antv/l7-utils");
var import_lodash = require("lodash");
var import_camera = __toESM(require("./camera"));
var import_l7 = require("./css/l7.css");
var import_lng_lat = __toESM(require("./geo/lng_lat"));
var import_lng_lat_bounds = __toESM(require("./geo/lng_lat_bounds"));
var import_point = __toESM(require("./geo/point"));
var import_handler_manager = __toESM(require("./handler/handler_manager"));
var import_util = require("./util");
var import_performance = require("./utils/performance");
var import_task_queue = __toESM(require("./utils/task_queue"));
var defaultMinZoom = -2;
var defaultMaxZoom = 22;
var defaultMinPitch = 0;
var defaultMaxPitch = 60;
var DefaultOptions = {
  hash: false,
  zoom: -1,
  center: [112, 32],
  pitch: 0,
  bearing: 0,
  interactive: true,
  minZoom: defaultMinZoom,
  maxZoom: defaultMaxZoom,
  minPitch: defaultMinPitch,
  maxPitch: defaultMaxPitch,
  scrollZoom: true,
  boxZoom: true,
  dragRotate: true,
  dragPan: true,
  keyboard: true,
  doubleClickZoom: true,
  touchZoomRotate: true,
  touchPitch: true,
  bearingSnap: 7,
  clickTolerance: 3,
  pitchWithRotate: true,
  trackResize: true,
  renderWorldCopies: true,
  pitchEnabled: true,
  rotateEnabled: true
};
var EarthMap = class extends import_camera.default {
  constructor(options) {
    super((0, import_lodash.merge)({}, DefaultOptions, options));
    this.renderTaskQueue = new import_task_queue.default();
    this.trackResize = true;
    this.onWindowOnline = () => {
      this.update();
    };
    this.onWindowResize = (event) => {
      if (this.trackResize) {
        this.resize({ originalEvent: event }).update();
      }
    };
    this.initContainer();
    this.resize();
    this.handlers = new import_handler_manager.default(this, this.options);
    if (typeof window !== "undefined") {
      window.addEventListener("online", this.onWindowOnline, false);
      window.addEventListener("resize", this.onWindowResize, false);
      window.addEventListener("orientationchange", this.onWindowResize, false);
    }
  }
  resize(eventData) {
    const dimensions = this.containerDimensions();
    const width = dimensions[0];
    const height = dimensions[1];
    this.transform.resize(width, height);
    const fireMoving = !this.moving;
    if (fireMoving) {
      this.stop();
      this.emit("movestart", new Event("movestart", eventData));
      this.emit("move", new Event("move", eventData));
    }
    this.emit("resize", new Event("resize", eventData));
    if (fireMoving) {
      this.emit("moveend", new Event("moveend", eventData));
    }
    return this;
  }
  getContainer() {
    return this.container;
  }
  getCanvas() {
    return this.canvas;
  }
  getCanvasContainer() {
    return this.canvasContainer;
  }
  project(lngLat) {
    return this.transform.locationPoint(import_lng_lat.default.convert(lngLat));
  }
  unproject(point) {
    return this.transform.pointLocation(import_point.default.convert(point));
  }
  getBounds() {
    return this.transform.getBounds();
  }
  getMaxBounds() {
    return this.transform.getMaxBounds();
  }
  setMaxBounds(bounds) {
    this.transform.setMaxBounds(import_lng_lat_bounds.default.convert(bounds));
  }
  setStyle(style) {
    return;
  }
  setMinZoom(minZoom) {
    minZoom = minZoom === null || minZoom === void 0 ? defaultMinZoom : minZoom;
    if (minZoom >= defaultMinZoom && minZoom <= this.transform.maxZoom) {
      this.transform.minZoom = minZoom;
      if (this.getZoom() < minZoom) {
        this.setZoom(minZoom);
      }
      return this;
    } else {
      throw new Error(`minZoom must be between ${defaultMinZoom} and the current maxZoom, inclusive`);
    }
  }
  getMinZoom() {
    return this.transform.minZoom;
  }
  setMaxZoom(maxZoom) {
    maxZoom = maxZoom === null || maxZoom === void 0 ? defaultMaxZoom : maxZoom;
    if (maxZoom >= this.transform.minZoom) {
      this.transform.maxZoom = maxZoom;
      if (this.getZoom() > maxZoom) {
        this.setZoom(maxZoom);
      }
      return this;
    } else {
      throw new Error("maxZoom must be greater than the current minZoom");
    }
  }
  getMaxZoom() {
    return this.transform.maxZoom;
  }
  setMinPitch(minPitch) {
    minPitch = minPitch === null || minPitch === void 0 ? defaultMinPitch : minPitch;
    if (minPitch < defaultMinPitch) {
      throw new Error(`minPitch must be greater than or equal to ${defaultMinPitch}`);
    }
    if (minPitch >= defaultMinPitch && minPitch <= this.transform.maxPitch) {
      this.transform.minPitch = minPitch;
      if (this.getPitch() < minPitch) {
        this.setPitch(minPitch);
      }
      return this;
    } else {
      throw new Error(`minPitch must be between ${defaultMinPitch} and the current maxPitch, inclusive`);
    }
  }
  getMinPitch() {
    return this.transform.minPitch;
  }
  setMaxPitch(maxPitch) {
    maxPitch = maxPitch === null || maxPitch === void 0 ? defaultMaxPitch : maxPitch;
    if (maxPitch > defaultMaxPitch) {
      throw new Error(`maxPitch must be less than or equal to ${defaultMaxPitch}`);
    }
    if (maxPitch >= this.transform.minPitch) {
      this.transform.maxPitch = maxPitch;
      if (this.getPitch() > maxPitch) {
        this.setPitch(maxPitch);
      }
      return this;
    } else {
      throw new Error("maxPitch must be greater than the current minPitch");
    }
  }
  getMaxPitch() {
    return this.transform.maxPitch;
  }
  getRenderWorldCopies() {
    return this.transform.renderWorldCopies;
  }
  setRenderWorldCopies(renderWorldCopies) {
    this.transform.renderWorldCopies = !!renderWorldCopies;
  }
  remove() {
    if (this.frame) {
      this.frame.cancel();
      this.frame = null;
    }
    this.renderTaskQueue.clear();
  }
  requestRenderFrame(cb) {
    this.update();
    return this.renderTaskQueue.add(cb);
  }
  cancelRenderFrame(id) {
    return this.renderTaskQueue.remove(id);
  }
  triggerRepaint() {
    if (!this.frame) {
      this.frame = (0, import_util.renderframe)((paintStartTimeStamp) => {
        import_performance.PerformanceUtils.frame(paintStartTimeStamp);
        this.frame = null;
        this.update(paintStartTimeStamp);
      });
    }
  }
  update(time) {
    if (!this.frame) {
      this.frame = (0, import_util.renderframe)((paintStartTimeStamp) => {
        import_performance.PerformanceUtils.frame(paintStartTimeStamp);
        this.frame = null;
        this.renderTaskQueue.run(time);
      });
    }
  }
  initContainer() {
    if (typeof this.options.container === "string") {
      this.container = window.document.getElementById(this.options.container);
      if (!this.container) {
        throw new Error(`Container '${this.options.container}' not found.`);
      }
    } else if (this.options.container instanceof HTMLElement) {
      this.container = this.options.container;
    } else {
      throw new Error("Invalid type: 'container' must be a String or HTMLElement.");
    }
    const container = this.container;
    container.classList.add("l7-map");
    const canvasContainer = this.canvasContainer = import_l7_utils.DOM.create("div", "l7-canvas-container", container);
    if (this.options.interactive) {
      canvasContainer.classList.add("l7-interactive");
    }
  }
  containerDimensions() {
    let width = 0;
    let height = 0;
    if (this.container) {
      width = this.container.clientWidth || 400;
      height = this.container.clientHeight || 300;
    }
    return [width, height];
  }
  resizeCanvas(width, height) {
    const pixelRatio = import_l7_utils.DOM.DPR || 1;
    this.canvas.width = pixelRatio * width;
    this.canvas.height = pixelRatio * height;
    this.canvas.style.width = `${width}px`;
    this.canvas.style.height = `${height}px`;
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  EarthMap
});
