var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target, mod));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/geo/transform.ts
var transform_exports = {};
__export(transform_exports, {
  EXTENT: () => EXTENT,
  default: () => Transform
});
module.exports = __toCommonJS(transform_exports);
var import_l7_utils = require("@antv/l7-utils");
var import_gl_matrix = require("gl-matrix");
var import_point = __toESM(require("../geo/point"));
var import_util = require("../util");
var import_edge_insets = __toESM(require("./edge_insets"));
var import_lng_lat = __toESM(require("./lng_lat"));
var import_lng_lat_bounds = __toESM(require("./lng_lat_bounds"));
var import_mercator = __toESM(require("./mercator"));
var EXTENT = 8192;
var Transform = class {
  get minZoom() {
    return this._minZoom;
  }
  set minZoom(zoom) {
    if (this._minZoom === zoom) {
      return;
    }
    this._minZoom = zoom;
    this.zoom = Math.max(this.zoom, zoom);
  }
  get maxZoom() {
    return this._maxZoom;
  }
  set maxZoom(zoom) {
    if (this._maxZoom === zoom) {
      return;
    }
    this._maxZoom = zoom;
    this.zoom = Math.min(this.zoom, zoom);
  }
  get minPitch() {
    return this._minPitch;
  }
  set minPitch(pitch) {
    if (this._minPitch === pitch) {
      return;
    }
    this._minPitch = pitch;
    this._pitch = Math.max(this._pitch, pitch);
  }
  get maxPitch() {
    return this._maxPitch;
  }
  set maxPitch(pitch) {
    if (this._maxPitch === pitch) {
      return;
    }
    this._maxPitch = pitch;
    this._pitch = Math.min(this._pitch, pitch);
  }
  get renderWorldCopies() {
    return this._renderWorldCopies;
  }
  set renderWorldCopies(renderWorldCopies) {
    if (renderWorldCopies === void 0) {
      renderWorldCopies = true;
    } else if (renderWorldCopies === null) {
      renderWorldCopies = false;
    }
    this._renderWorldCopies = renderWorldCopies;
  }
  get worldSize() {
    return this.tileSize * this.scale;
  }
  get centerOffset() {
    return this.centerPoint._sub(this.size._div(2));
  }
  get size() {
    return new import_point.default(this.width, this.height);
  }
  get bearing() {
    return -this.angle / Math.PI * 180;
  }
  set bearing(bearing) {
    const b = -(0, import_util.wrap)(bearing, -180, 180) * Math.PI / 180;
    if (this.angle === b) {
      return;
    }
    this.unmodified = false;
    this.angle = b;
    this.calcMatrices();
    this.rotationMatrix = import_gl_matrix.mat2.create();
    import_gl_matrix.mat2.rotate(this.rotationMatrix, this.rotationMatrix, this.angle);
  }
  get pitch() {
    return this._pitch / Math.PI * 180;
  }
  set pitch(pitch) {
    const p = (0, import_util.clamp)(pitch, this._minPitch, this._maxPitch) / 180 * Math.PI;
    if (this._pitch === p) {
      return;
    }
    this.unmodified = false;
    this._pitch = p;
    this.calcMatrices();
  }
  get fov() {
    return this._fov / Math.PI * 180;
  }
  set fov(fov) {
    fov = Math.max(0.01, Math.min(60, fov));
    if (this._fov === fov) {
      return;
    }
    this.unmodified = false;
    this._fov = fov / 180 * Math.PI;
    this.calcMatrices();
  }
  get zoom() {
    return this._zoom;
  }
  set zoom(zoom) {
    const z = Math.min(Math.max(zoom, this._minZoom), this._maxZoom);
    if (this._zoom === z) {
      return;
    }
    this.unmodified = false;
    this._zoom = z;
    this.scale = this.zoomScale(z);
    this.tileZoom = Math.floor(z);
    this.zoomFraction = z - this.tileZoom;
    this.constrain();
    this.calcMatrices();
  }
  get center() {
    return this._center;
  }
  set center(center) {
    if (center.lat === this._center.lat && center.lng === this._center.lng) {
      return;
    }
    this.unmodified = false;
    this._center = center;
    this.constrain();
    this.calcMatrices();
  }
  get padding() {
    return this.edgeInsets.toJSON();
  }
  set padding(padding) {
    if (this.edgeInsets.equals(padding)) {
      return;
    }
    this.unmodified = false;
    this.edgeInsets.interpolate(this.edgeInsets, padding, 1);
    this.calcMatrices();
  }
  get centerPoint() {
    return this.edgeInsets.getCenter(this.width, this.height);
  }
  get point() {
    return this.project(this.center);
  }
  constructor(minZoom, maxZoom, minPitch, maxPitch, renderWorldCopies) {
    this.tileSize = 512;
    this.maxValidLatitude = 85.051129;
    this._renderWorldCopies = renderWorldCopies === void 0 ? true : renderWorldCopies;
    this._minZoom = minZoom || 0;
    this._maxZoom = maxZoom || 22;
    this._minPitch = minPitch === void 0 || minPitch === null ? 0 : minPitch;
    this._maxPitch = maxPitch === void 0 || maxPitch === null ? 60 : maxPitch;
    this.setMaxBounds();
    this.width = 0;
    this.height = 0;
    this._center = new import_lng_lat.default(0, 0);
    this.zoom = 0;
    this.angle = 0;
    this._fov = 0.6435011087932844;
    this._pitch = 0;
    this.unmodified = true;
    this.edgeInsets = new import_edge_insets.default();
    this.posMatrixCache = {};
    this.alignedPosMatrixCache = {};
  }
  clone() {
    const clone = new Transform(this._minZoom, this._maxZoom, this._minPitch, this._maxPitch, this._renderWorldCopies);
    clone.tileSize = this.tileSize;
    clone.latRange = this.latRange;
    clone.width = this.width;
    clone.height = this.height;
    clone.center = this._center;
    clone.zoom = this.zoom;
    clone.angle = this.angle;
    clone.fov = this._fov;
    clone.pitch = this._pitch;
    clone.unmodified = this.unmodified;
    clone.edgeInsets = this.edgeInsets.clone();
    clone.calcMatrices();
    return clone;
  }
  isPaddingEqual(padding) {
    return this.edgeInsets.equals(padding);
  }
  interpolatePadding(start, target, t) {
    this.unmodified = false;
    this.edgeInsets.interpolate(start, target, t);
    this.constrain();
    this.calcMatrices();
  }
  coveringZoomLevel(options) {
    const z = (options.roundZoom ? Math.round : Math.floor)(this.zoom + this.scaleZoom(this.tileSize / options.tileSize));
    return Math.max(0, z);
  }
  resize(width, height) {
    this.width = width;
    this.height = height;
    this.pixelsToGLUnits = [2 / width, -2 / height];
    this.constrain();
    this.calcMatrices();
  }
  zoomScale(zoom) {
    return Math.pow(2, zoom);
  }
  scaleZoom(scale) {
    return Math.log(scale) / Math.LN2;
  }
  project(lnglat) {
    const lat = (0, import_util.clamp)(lnglat.lat, -this.maxValidLatitude, this.maxValidLatitude);
    return new import_point.default((0, import_mercator.mercatorXfromLng)(lnglat.lng) * this.worldSize, (0, import_mercator.mercatorYfromLat)(lat) * this.worldSize);
  }
  unproject(point) {
    return new import_mercator.default(point.x / this.worldSize, point.y / this.worldSize).toLngLat();
  }
  setLocationAtPoint(lnglat, point) {
    const a = this.pointCoordinate(point);
    const b = this.pointCoordinate(this.centerPoint);
    const loc = this.locationCoordinate(lnglat);
    const newCenter = new import_mercator.default(loc.x - (a.x - b.x), loc.y - (a.y - b.y));
    this.center = this.coordinateLocation(newCenter);
    if (this._renderWorldCopies) {
      this.center = this.center.wrap();
    }
  }
  pointCoordinate(p) {
    const targetZ = 0;
    const coord0 = new Float64Array([p.x, p.y, 0, 1]);
    const coord1 = new Float64Array([p.x, p.y, 1, 1]);
    import_gl_matrix.vec4.transformMat4(coord0, coord0, this.pixelMatrixInverse);
    import_gl_matrix.vec4.transformMat4(coord1, coord1, this.pixelMatrixInverse);
    const w0 = coord0[3];
    const w1 = coord1[3];
    const x0 = coord0[0] / w0;
    const x1 = coord1[0] / w1;
    const y0 = coord0[1] / w0;
    const y1 = coord1[1] / w1;
    const z0 = coord0[2] / w0;
    const z1 = coord1[2] / w1;
    const t = z0 === z1 ? 0 : (targetZ - z0) / (z1 - z0);
    return new import_mercator.default((0, import_util.interpolate)(x0, x1, t) / this.worldSize, (0, import_util.interpolate)(y0, y1, t) / this.worldSize);
  }
  getBounds() {
    return new import_lng_lat_bounds.default().extend(this.pointLocation(new import_point.default(0, 0))).extend(this.pointLocation(new import_point.default(this.width, 0))).extend(this.pointLocation(new import_point.default(this.width, this.height))).extend(this.pointLocation(new import_point.default(0, this.height)));
  }
  getMaxBounds() {
    if (!this.latRange || this.latRange.length !== 2 || !this.lngRange || this.lngRange.length !== 2) {
      return null;
    }
    return new import_lng_lat_bounds.default([this.lngRange[0], this.latRange[0]], [this.lngRange[1], this.latRange[1]]);
  }
  setMaxBounds(bounds) {
    if (bounds) {
      this.lngRange = [bounds.getWest(), bounds.getEast()];
      this.latRange = [bounds.getSouth(), bounds.getNorth()];
      this.constrain();
    } else {
      this.lngRange = void 0;
      this.latRange = [-this.maxValidLatitude, this.maxValidLatitude];
    }
  }
  customLayerMatrix() {
    return this.mercatorMatrix.slice();
  }
  maxPitchScaleFactor() {
    if (!this.pixelMatrixInverse) {
      return 1;
    }
    const coord = this.pointCoordinate(new import_point.default(0, 0));
    const p = new Float32Array([
      coord.x * this.worldSize,
      coord.y * this.worldSize,
      0,
      1
    ]);
    const topPoint = import_gl_matrix.vec4.transformMat4(p, p, this.pixelMatrix);
    return topPoint[3] / this.cameraToCenterDistance;
  }
  getCameraPoint() {
    const pitch = this._pitch;
    const yOffset = Math.tan(pitch) * (this.cameraToCenterDistance || 1);
    return this.centerPoint.add(new import_point.default(0, yOffset));
  }
  getCameraQueryGeometry(queryGeometry) {
    const c = this.getCameraPoint();
    if (queryGeometry.length === 1) {
      return [queryGeometry[0], c];
    } else {
      let minX = c.x;
      let minY = c.y;
      let maxX = c.x;
      let maxY = c.y;
      for (const p of queryGeometry) {
        minX = Math.min(minX, p.x);
        minY = Math.min(minY, p.y);
        maxX = Math.max(maxX, p.x);
        maxY = Math.max(maxY, p.y);
      }
      return [
        new import_point.default(minX, minY),
        new import_point.default(maxX, minY),
        new import_point.default(maxX, maxY),
        new import_point.default(minX, maxY),
        new import_point.default(minX, minY)
      ];
    }
  }
  coordinatePoint(coord) {
    const p = import_gl_matrix.vec4.fromValues(coord.x * this.worldSize, coord.y * this.worldSize, 0, 1);
    import_gl_matrix.vec4.transformMat4(p, p, this.pixelMatrix);
    return new import_point.default(p[0] / p[3], p[1] / p[3]);
  }
  locationPoint(lnglat) {
    return this.coordinatePoint(this.locationCoordinate(lnglat));
  }
  pointLocation(p) {
    return this.coordinateLocation(this.pointCoordinate(p));
  }
  locationCoordinate(lnglat) {
    return import_mercator.default.fromLngLat(lnglat);
  }
  coordinateLocation(coord) {
    return coord.toLngLat();
  }
  getProjectionMatrix() {
    return this.projMatrix;
  }
  constrain() {
    if (!this.center || !this.width || !this.height || this.constraining) {
      return;
    }
    this.constraining = true;
    let minY = -90;
    let maxY = 90;
    let minX = -180;
    let maxX = 180;
    let sy;
    let sx;
    let x2;
    let y2;
    const size = this.size;
    const unmodified = this.unmodified;
    if (this.latRange) {
      const latRange = this.latRange;
      minY = (0, import_mercator.mercatorYfromLat)(latRange[1]) * this.worldSize;
      maxY = (0, import_mercator.mercatorYfromLat)(latRange[0]) * this.worldSize;
      sy = maxY - minY < size.y ? size.y / (maxY - minY) : 0;
    }
    if (this.lngRange) {
      const lngRange = this.lngRange;
      minX = (0, import_mercator.mercatorXfromLng)(lngRange[0]) * this.worldSize;
      maxX = (0, import_mercator.mercatorXfromLng)(lngRange[1]) * this.worldSize;
      sx = maxX - minX < size.x ? size.x / (maxX - minX) : 0;
    }
    const point = this.point;
    const s = Math.max(sx || 0, sy || 0);
    if (s) {
      this.center = this.unproject(new import_point.default(sx ? (maxX + minX) / 2 : point.x, sy ? (maxY + minY) / 2 : point.y));
      if (import_l7_utils.isMini) {
        this.zoom = Math.max(this.zoom, Math.max(-1, this.minZoom));
      } else {
        this.zoom += this.scaleZoom(s);
      }
      this.unmodified = unmodified;
      this.constraining = false;
      return;
    }
    if (this.latRange) {
      const y = point.y;
      const h2 = size.y / 2;
      if (y - h2 < minY) {
        y2 = minY + h2;
      }
      if (y + h2 > maxY) {
        y2 = maxY - h2;
      }
    }
    if (this.lngRange) {
      const x = point.x;
      const w2 = size.x / 2;
      if (x - w2 < minX) {
        x2 = minX + w2;
      }
      if (x + w2 > maxX) {
        x2 = maxX - w2;
      }
    }
    if (x2 !== void 0 || y2 !== void 0) {
      this.center = this.unproject(new import_point.default(x2 !== void 0 ? x2 : point.x, y2 !== void 0 ? y2 : point.y));
    }
    this.unmodified = unmodified;
    this.constraining = false;
  }
  calcMatrices() {
    if (!this.height) {
      return;
    }
    const halfFov = this._fov / 2;
    const offset = this.centerOffset;
    this.cameraToCenterDistance = 0.5 / Math.tan(halfFov) * this.height;
    const groundAngle = Math.PI / 2 + this._pitch;
    const fovAboveCenter = this._fov * (0.5 + offset.y / this.height);
    const topHalfSurfaceDistance = Math.sin(fovAboveCenter) * this.cameraToCenterDistance / Math.sin((0, import_util.clamp)(Math.PI - groundAngle - fovAboveCenter, 0.01, Math.PI - 0.01));
    const point = this.point;
    const x = point.x;
    const y = point.y;
    const furthestDistance = Math.cos(Math.PI / 2 - this._pitch) * topHalfSurfaceDistance + this.cameraToCenterDistance;
    const farZ = furthestDistance * 1.01;
    const nearZ = this.height / 50;
    let m = new Float64Array(16);
    import_gl_matrix.mat4.perspective(m, this._fov, this.width / this.height, nearZ, farZ);
    m[8] = -offset.x * 2 / this.width;
    m[9] = offset.y * 2 / this.height;
    import_gl_matrix.mat4.scale(m, m, [1, -1, 1]);
    import_gl_matrix.mat4.translate(m, m, [0, 0, -this.cameraToCenterDistance]);
    import_gl_matrix.mat4.rotateX(m, m, this._pitch);
    import_gl_matrix.mat4.rotateZ(m, m, this.angle);
    import_gl_matrix.mat4.translate(m, m, [-x, -y, 0]);
    this.mercatorMatrix = import_gl_matrix.mat4.scale([], m, [
      this.worldSize,
      this.worldSize,
      this.worldSize
    ]);
    import_gl_matrix.mat4.scale(m, m, [
      1,
      1,
      (0, import_mercator.mercatorZfromAltitude)(1, this.center.lat) * this.worldSize,
      1
    ]);
    this.projMatrix = m;
    this.invProjMatrix = import_gl_matrix.mat4.invert([], this.projMatrix);
    const xShift = this.width % 2 / 2;
    const yShift = this.height % 2 / 2;
    const angleCos = Math.cos(this.angle);
    const angleSin = Math.sin(this.angle);
    const dx = x - Math.round(x) + angleCos * xShift + angleSin * yShift;
    const dy = y - Math.round(y) + angleCos * yShift + angleSin * xShift;
    const alignedM = new Float64Array(m);
    import_gl_matrix.mat4.translate(alignedM, alignedM, [
      dx > 0.5 ? dx - 1 : dx,
      dy > 0.5 ? dy - 1 : dy,
      0
    ]);
    this.alignedProjMatrix = alignedM;
    m = import_gl_matrix.mat4.create();
    import_gl_matrix.mat4.scale(m, m, [this.width / 2, -this.height / 2, 1]);
    import_gl_matrix.mat4.translate(m, m, [1, -1, 0]);
    this.labelPlaneMatrix = m;
    m = import_gl_matrix.mat4.create();
    import_gl_matrix.mat4.scale(m, m, [1, -1, 1]);
    import_gl_matrix.mat4.translate(m, m, [-1, -1, 0]);
    import_gl_matrix.mat4.scale(m, m, [2 / this.width, 2 / this.height, 1]);
    this.glCoordMatrix = m;
    this.pixelMatrix = import_gl_matrix.mat4.multiply(new Float64Array(16), this.labelPlaneMatrix, this.projMatrix);
    m = import_gl_matrix.mat4.invert(new Float64Array(16), this.pixelMatrix);
    if (!m) {
      throw new Error("failed to invert matrix");
    }
    this.pixelMatrixInverse = m;
    this.posMatrixCache = {};
    this.alignedPosMatrixCache = {};
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  EXTENT
});
