var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target, mod));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/handler/box_zoom.ts
var box_zoom_exports = {};
__export(box_zoom_exports, {
  default: () => box_zoom_default
});
module.exports = __toCommonJS(box_zoom_exports);
var import_dom = __toESM(require("../utils/dom"));
var import_event = require("./events/event");
var BoxZoomHandler = class {
  constructor(map, options) {
    this.map = map;
    this.el = map.getCanvasContainer();
    this.container = map.getContainer();
    this.clickTolerance = options.clickTolerance || 1;
  }
  isEnabled() {
    return !!this.enabled;
  }
  isActive() {
    return !!this.active;
  }
  enable() {
    if (this.isEnabled()) {
      return;
    }
    this.enabled = true;
  }
  disable() {
    if (!this.isEnabled()) {
      return;
    }
    this.enabled = false;
  }
  mousedown(e, point) {
    if (!this.isEnabled()) {
      return;
    }
    if (!(e.shiftKey && e.button === 0)) {
      return;
    }
    import_dom.default.disableDrag();
    this.startPos = this.lastPos = point;
    this.active = true;
  }
  mousemoveWindow(e, point) {
    if (!this.active) {
      return;
    }
    const pos = point;
    if (this.lastPos.equals(pos) || !this.box && pos.dist(this.startPos) < this.clickTolerance) {
      return;
    }
    const p0 = this.startPos;
    this.lastPos = pos;
    if (!this.box) {
      this.box = import_dom.default.create("div", "l7-boxzoom", this.container);
      this.container.classList.add("l7-crosshair");
      this.fireEvent("boxzoomstart", e);
    }
    const minX = Math.min(p0.x, pos.x);
    const maxX = Math.max(p0.x, pos.x);
    const minY = Math.min(p0.y, pos.y);
    const maxY = Math.max(p0.y, pos.y);
    import_dom.default.setTransform(this.box, `translate(${minX}px,${minY}px)`);
    if (this.box) {
      this.box.style.width = `${maxX - minX}px`;
      this.box.style.height = `${maxY - minY}px`;
    }
  }
  mouseupWindow(e, point) {
    if (!this.active) {
      return;
    }
    if (e.button !== 0) {
      return;
    }
    const p0 = this.startPos;
    const p1 = point;
    this.reset();
    import_dom.default.suppressClick();
    if (p0.x === p1.x && p0.y === p1.y) {
      this.fireEvent("boxzoomcancel", e);
    } else {
      this.map.emit("boxzoomend", new import_event.Event("boxzoomend", { originalEvent: e }));
      return {
        cameraAnimation: (map) => map.fitScreenCoordinates(p0, p1, this.map.getBearing(), {
          linear: true
        })
      };
    }
  }
  keydown(e) {
    if (!this.active) {
      return;
    }
    if (e.keyCode === 27) {
      this.reset();
      this.fireEvent("boxzoomcancel", e);
    }
  }
  reset() {
    this.active = false;
    this.container.classList.remove("l7-crosshair");
    if (this.box) {
      import_dom.default.remove(this.box);
      this.box = null;
    }
    import_dom.default.enableDrag();
    delete this.startPos;
    delete this.lastPos;
  }
  fireEvent(type, e) {
    return this.map.emit(type, new import_event.Event(type, { originalEvent: e }));
  }
};
var box_zoom_default = BoxZoomHandler;
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {});
