var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target, mod));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/handler/handler_inertia.ts
var handler_inertia_exports = {};
__export(handler_inertia_exports, {
  default: () => HandlerInertia
});
module.exports = __toCommonJS(handler_inertia_exports);
var import_point = __toESM(require("../geo/point"));
var import_merge = __toESM(require("lodash/merge"));
var import_util = require("../util");
var defaultInertiaOptions = {
  linearity: 0.3,
  easing: (0, import_util.bezier)(0, 0, 0.3, 1)
};
var defaultPanInertiaOptions = (0, import_merge.default)({
  deceleration: 2500,
  maxSpeed: 1400
}, defaultInertiaOptions);
var defaultZoomInertiaOptions = (0, import_merge.default)({
  deceleration: 20,
  maxSpeed: 1400
}, defaultInertiaOptions);
var defaultBearingInertiaOptions = (0, import_merge.default)({
  deceleration: 1e3,
  maxSpeed: 360
}, defaultInertiaOptions);
var defaultPitchInertiaOptions = (0, import_merge.default)({
  deceleration: 1e3,
  maxSpeed: 90
}, defaultInertiaOptions);
var HandlerInertia = class {
  constructor(map) {
    this.map = map;
    this.clear();
  }
  clear() {
    this.inertiaBuffer = [];
  }
  record(settings) {
    this.drainInertiaBuffer();
    this.inertiaBuffer.push({ time: (0, import_util.now)(), settings });
  }
  drainInertiaBuffer() {
    const inertia = this.inertiaBuffer;
    const nowTime = (0, import_util.now)();
    const cutoff = 160;
    while (inertia.length > 0 && nowTime - inertia[0].time > cutoff) {
      inertia.shift();
    }
  }
  onMoveEnd(panInertiaOptions) {
    this.drainInertiaBuffer();
    if (this.inertiaBuffer.length < 2) {
      return;
    }
    const deltas = {
      zoom: 0,
      bearing: 0,
      pitch: 0,
      pan: new import_point.default(0, 0),
      pinchAround: void 0,
      around: void 0
    };
    for (const { settings } of this.inertiaBuffer) {
      deltas.zoom += settings.zoomDelta || 0;
      deltas.bearing += settings.bearingDelta || 0;
      deltas.pitch += settings.pitchDelta || 0;
      if (settings.panDelta) {
        deltas.pan._add(settings.panDelta);
      }
      if (settings.around) {
        deltas.around = settings.around;
      }
      if (settings.pinchAround) {
        deltas.pinchAround = settings.pinchAround;
      }
    }
    const lastEntry = this.inertiaBuffer[this.inertiaBuffer.length - 1];
    const duration = lastEntry.time - this.inertiaBuffer[0].time;
    const easeOptions = {};
    if (deltas.pan.mag()) {
      const result = calculateEasing(deltas.pan.mag(), duration, (0, import_merge.default)({}, defaultPanInertiaOptions, panInertiaOptions || {}));
      easeOptions.offset = deltas.pan.mult(result.amount / deltas.pan.mag());
      easeOptions.center = this.map.transform.center;
      extendDuration(easeOptions, result);
    }
    if (deltas.zoom) {
      const result = calculateEasing(deltas.zoom, duration, defaultZoomInertiaOptions);
      easeOptions.zoom = this.map.transform.zoom + result.amount;
      extendDuration(easeOptions, result);
    }
    if (deltas.bearing) {
      const result = calculateEasing(deltas.bearing, duration, defaultBearingInertiaOptions);
      easeOptions.bearing = this.map.transform.bearing + (0, import_util.clamp)(result.amount, -179, 179);
      extendDuration(easeOptions, result);
    }
    if (deltas.pitch) {
      const result = calculateEasing(deltas.pitch, duration, defaultPitchInertiaOptions);
      easeOptions.pitch = this.map.transform.pitch + result.amount;
      extendDuration(easeOptions, result);
    }
    if (easeOptions.zoom || easeOptions.bearing) {
      const last = deltas.pinchAround === void 0 ? deltas.around : deltas.pinchAround;
      easeOptions.around = last ? this.map.unproject(last) : this.map.getCenter();
    }
    this.clear();
    return (0, import_merge.default)(easeOptions, {
      noMoveStart: true
    });
  }
};
function extendDuration(easeOptions, result) {
  if (!easeOptions.duration || easeOptions.duration < result.duration) {
    easeOptions.duration = result.duration;
    easeOptions.easing = result.easing;
  }
}
function calculateEasing(amount, inertiaDuration, inertiaOptions) {
  const { maxSpeed, linearity, deceleration } = inertiaOptions;
  const speed = (0, import_util.clamp)(amount * linearity / (inertiaDuration / 1e3), -maxSpeed, maxSpeed);
  const duration = Math.abs(speed) / (deceleration * linearity);
  return {
    easing: inertiaOptions.easing,
    duration: duration * 1e3,
    amount: speed * (duration / 2)
  };
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {});
