var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target, mod));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/handler/handler_manager.ts
var handler_manager_exports = {};
__export(handler_manager_exports, {
  default: () => handler_manager_default
});
module.exports = __toCommonJS(handler_manager_exports);
var import_l7_utils = require("@antv/l7-utils");
var import_lodash = require("lodash");
var import_point = __toESM(require("../geo/point"));
var import_dom = __toESM(require("../utils/dom"));
var import_blockable_map_event = __toESM(require("./blockable_map_event"));
var import_box_zoom = __toESM(require("./box_zoom"));
var import_click_zoom = __toESM(require("./click_zoom"));
var import_event = require("./events/event");
var import_render_event = __toESM(require("./events/render_event"));
var import_handler_inertia = __toESM(require("./handler_inertia"));
var import_keyboard = __toESM(require("./keyboard"));
var import_map_event = __toESM(require("./map_event"));
var import_mouse = require("./mouse");
var import_scroll_zoom = __toESM(require("./scroll_zoom"));
var import_dblclick_zoom = __toESM(require("./shim/dblclick_zoom"));
var import_drag_pan = __toESM(require("./shim/drag_pan"));
var import_drag_rotate = __toESM(require("./shim/drag_rotate"));
var import_touch_zoom_rotate = __toESM(require("./shim/touch_zoom_rotate"));
var import_tap_drag_zoom = __toESM(require("./tap/tap_drag_zoom"));
var import_tap_zoom = __toESM(require("./tap/tap_zoom"));
var import_touch = require("./touch");
var isMoving = (p) => p.zoom || p.drag || p.pitch || p.rotate;
function hasChange(result) {
  return result.panDelta && result.panDelta.mag() || result.zoomDelta || result.bearingDelta || result.pitchDelta;
}
var HandlerManager = class {
  constructor(map, options) {
    this.handleWindowEvent = (e) => {
      this.handleEvent(e, `${e.type}Window`);
    };
    this.handleEvent = (e, eventName) => {
      if (e.type === "blur") {
        this.stop();
        return;
      }
      this.updatingCamera = true;
      const inputEvent = e.type === "renderFrame" ? void 0 : e;
      const mergedIHandlerResult = { needsRenderFrame: false };
      const eventsInProgress = {};
      const activeHandlers = {};
      const mapTouches = e.touches ? this.getMapTouches(e.touches) : void 0;
      const points = mapTouches ? import_dom.default.touchPos(this.el, mapTouches) : import_dom.default.mousePos(this.el, e);
      for (const { handlerName, handler, allowed } of this.handlers) {
        if (!handler.isEnabled()) {
          continue;
        }
        let data;
        if (this.blockedByActive(activeHandlers, allowed, handlerName)) {
          handler.reset();
        } else {
          const handerName = eventName || e.type;
          if (handler && handler[handerName]) {
            data = handler[handerName](e, points, mapTouches);
            this.mergeIHandlerResult(mergedIHandlerResult, eventsInProgress, data, handlerName, inputEvent);
            if (data && data.needsRenderFrame) {
              this.triggerRenderFrame();
            }
          }
        }
        if (data || handler.isActive()) {
          activeHandlers[handlerName] = handler;
        }
      }
      const deactivatedHandlers = {};
      for (const name in this.previousActiveHandlers) {
        if (!activeHandlers[name]) {
          deactivatedHandlers[name] = inputEvent;
        }
      }
      this.previousActiveHandlers = activeHandlers;
      if (Object.keys(deactivatedHandlers).length || hasChange(mergedIHandlerResult)) {
        this.changes.push([
          mergedIHandlerResult,
          eventsInProgress,
          deactivatedHandlers
        ]);
        this.triggerRenderFrame();
      }
      if (Object.keys(activeHandlers).length || hasChange(mergedIHandlerResult)) {
        this.map.stop(true);
      }
      this.updatingCamera = false;
      const { cameraAnimation } = mergedIHandlerResult;
      if (cameraAnimation) {
        this.inertia.clear();
        this.fireEvents({}, {});
        this.changes = [];
        cameraAnimation(this.map);
      }
    };
    this.map = map;
    this.el = this.map.getCanvasContainer();
    this.handlers = [];
    this.handlersById = {};
    this.changes = [];
    this.inertia = new import_handler_inertia.default(map);
    this.bearingSnap = options.bearingSnap;
    this.rotateEnabled = options.rotateEnabled;
    this.pitchEnabled = options.pitchEnabled;
    this.previousActiveHandlers = {};
    this.eventsInProgress = {};
    this.addDefaultHandlers(options);
    const el = this.el;
    this.listeners = [];
    if (!import_l7_utils.isMini) {
      this.listeners = [
        [el, "touchstart", { passive: false }],
        [el, "touchmove", { passive: false }],
        [el, "touchend", void 0],
        [el, "touchcancel", void 0],
        [el, "mousedown", void 0],
        [el, "mousemove", void 0],
        [el, "mouseup", void 0],
        [window.document, "mousemove", { capture: true }],
        [window.document, "mouseup", void 0],
        [el, "mouseover", void 0],
        [el, "mouseout", void 0],
        [el, "dblclick", void 0],
        [el, "click", void 0],
        [el, "keydown", { capture: false }],
        [el, "keyup", void 0],
        [el, "wheel", { passive: false }],
        [el, "contextmenu", void 0],
        [window, "blur", void 0]
      ];
      for (const [target, type, listenerOptions] of this.listeners) {
        import_dom.default.addEventListener(target, type, target === window.document ? this.handleWindowEvent : this.handleEvent, listenerOptions);
      }
    } else {
      import_l7_utils.$window.document.addEventListener("touchstart", (e) => {
        this.handleEvent(e);
      }, {});
      import_l7_utils.$window.document.addEventListener("touchmove", (e) => {
        this.handleEvent(e);
      }, {});
      import_l7_utils.$window.document.addEventListener("touchend", (e) => {
        this.handleEvent(e);
      }, {});
      import_l7_utils.$window.document.addEventListener("touchcancel", (e) => {
        this.handleEvent(e);
      }, {});
    }
  }
  destroy() {
    for (const [target, type, listenerOptions] of this.listeners) {
      import_dom.default.removeEventListener(target, type, target === window.document ? this.handleWindowEvent : this.handleEvent, listenerOptions);
    }
  }
  stop() {
    if (this.updatingCamera) {
      return;
    }
    for (const { handler } of this.handlers) {
      handler.reset();
    }
    this.inertia.clear();
    this.fireEvents({}, {});
    this.changes = [];
  }
  isActive() {
    for (const { handler } of this.handlers) {
      if (handler.isActive()) {
        return true;
      }
    }
    return false;
  }
  isZooming() {
    return !!this.eventsInProgress.zoom || this.map.scrollZoom.isZooming();
  }
  isRotating() {
    return !!this.eventsInProgress.rotate;
  }
  isMoving() {
    return Boolean(isMoving(this.eventsInProgress)) || this.isZooming();
  }
  mergeIHandlerResult(mergedIHandlerResult, eventsInProgress, HandlerResult, name, e) {
    if (!HandlerResult) {
      return;
    }
    (0, import_lodash.merge)(mergedIHandlerResult, HandlerResult);
    const eventData = {
      handlerName: name,
      originalEvent: HandlerResult.originalEvent || e
    };
    if (HandlerResult.zoomDelta !== void 0) {
      eventsInProgress.zoom = eventData;
    }
    if (HandlerResult.panDelta !== void 0) {
      eventsInProgress.drag = eventData;
    }
    if (HandlerResult.pitchDelta !== void 0) {
      eventsInProgress.pitch = eventData;
    }
    if (HandlerResult.bearingDelta !== void 0) {
      eventsInProgress.rotate = eventData;
    }
  }
  triggerRenderFrame() {
    if (this.frameId === void 0) {
      this.frameId = this.map.requestRenderFrame((timeStamp) => {
        delete this.frameId;
        this.handleEvent(new import_render_event.default("renderFrame", { timeStamp }));
        this.applyChanges();
      });
    }
  }
  addDefaultHandlers(options) {
    const map = this.map;
    const el = map.getCanvasContainer();
    this.add("mapEvent", new import_map_event.default(map, options));
    const boxZoom = map.boxZoom = new import_box_zoom.default(map, options);
    this.add("boxZoom", boxZoom);
    const tapZoom = new import_tap_zoom.default();
    const clickZoom = new import_click_zoom.default();
    map.doubleClickZoom = new import_dblclick_zoom.default(clickZoom, tapZoom);
    this.add("tapZoom", tapZoom);
    this.add("clickZoom", clickZoom);
    const tapDragZoom = new import_tap_drag_zoom.default();
    this.add("tapDragZoom", tapDragZoom);
    const touchPitch = map.touchPitch = new import_touch.TouchPitchHandler();
    this.add("touchPitch", touchPitch);
    const mouseRotate = new import_mouse.MouseRotateHandler(options);
    const mousePitch = new import_mouse.MousePitchHandler(options);
    map.dragRotate = new import_drag_rotate.default(options, mouseRotate, mousePitch);
    this.add("mouseRotate", mouseRotate, ["mousePitch"]);
    this.add("mousePitch", mousePitch, ["mouseRotate"]);
    const mousePan = new import_mouse.MousePanHandler(options);
    const touchPan = new import_touch.TouchPanHandler(options);
    map.dragPan = new import_drag_pan.default(el, mousePan, touchPan);
    this.add("mousePan", mousePan);
    this.add("touchPan", touchPan, ["touchZoom", "touchRotate"]);
    const touchRotate = new import_touch.TouchRotateHandler();
    const touchZoom = new import_touch.TouchZoomHandler();
    map.touchZoomRotate = new import_touch_zoom_rotate.default(el, touchZoom, touchRotate, tapDragZoom);
    this.add("touchRotate", touchRotate, ["touchPan", "touchZoom"]);
    this.add("touchZoom", touchZoom, ["touchPan", "touchRotate"]);
    const scrollZoom = map.scrollZoom = new import_scroll_zoom.default(map, this);
    this.add("scrollZoom", scrollZoom, ["mousePan"]);
    const keyboard = map.keyboard = new import_keyboard.default();
    this.add("keyboard", keyboard);
    this.add("blockableMapEvent", new import_blockable_map_event.default(map));
    for (const name of [
      "boxZoom",
      "doubleClickZoom",
      "tapDragZoom",
      "touchPitch",
      "dragRotate",
      "dragPan",
      "touchZoomRotate",
      "scrollZoom",
      "keyboard"
    ]) {
      if (options.interactive && options[name]) {
        map[name].enable(options[name]);
      }
    }
  }
  add(handlerName, handler, allowed) {
    this.handlers.push({ handlerName, handler, allowed });
    this.handlersById[handlerName] = handler;
  }
  blockedByActive(activeHandlers, allowed, myName) {
    for (const name in activeHandlers) {
      if (name === myName) {
        continue;
      }
      if (!allowed || allowed.indexOf(name) < 0) {
        return true;
      }
    }
    return false;
  }
  getMapTouches(touches) {
    const mapTouches = [];
    for (const t of touches) {
      const target = t.target;
      if (import_l7_utils.isMini || this.el.contains(target)) {
        mapTouches.push(t);
      }
    }
    return mapTouches;
  }
  applyChanges() {
    const combined = {};
    const combinedEventsInProgress = {};
    const combinedDeactivatedHandlers = {};
    for (const [change, eventsInProgress, deactivatedHandlers] of this.changes) {
      if (change.panDelta) {
        combined.panDelta = (combined.panDelta || new import_point.default(0, 0))._add(change.panDelta);
      }
      if (change.zoomDelta) {
        combined.zoomDelta = (combined.zoomDelta || 0) + change.zoomDelta;
      }
      if (change.bearingDelta) {
        combined.bearingDelta = (combined.bearingDelta || 0) + change.bearingDelta;
      }
      if (change.pitchDelta) {
        combined.pitchDelta = (combined.pitchDelta || 0) + change.pitchDelta;
      }
      if (change.around !== void 0) {
        combined.around = change.around;
      }
      if (change.pinchAround !== void 0) {
        combined.pinchAround = change.pinchAround;
      }
      if (change.noInertia) {
        combined.noInertia = change.noInertia;
      }
      (0, import_lodash.merge)(combinedEventsInProgress, eventsInProgress);
      (0, import_lodash.merge)(combinedDeactivatedHandlers, deactivatedHandlers);
    }
    this.updateMapTransform(combined, combinedEventsInProgress, combinedDeactivatedHandlers);
    this.changes = [];
  }
  updateMapTransform(combinedResult, combinedEventsInProgress, deactivatedHandlers) {
    const map = this.map;
    const tr = map.transform;
    if (!hasChange(combinedResult)) {
      return this.fireEvents(combinedEventsInProgress, deactivatedHandlers);
    }
    const {
      panDelta,
      zoomDelta,
      bearingDelta,
      pitchDelta,
      pinchAround
    } = combinedResult;
    let { around } = combinedResult;
    if (pinchAround !== void 0) {
      around = pinchAround;
    }
    map.stop(true);
    around = around || map.transform.centerPoint;
    const loc = tr.pointLocation(panDelta ? around.sub(panDelta) : around);
    if (bearingDelta && this.rotateEnabled) {
      tr.bearing += bearingDelta;
    }
    if (pitchDelta && this.pitchEnabled) {
      tr.pitch += pitchDelta;
    }
    if (zoomDelta) {
      tr.zoom += zoomDelta;
    }
    tr.setLocationAtPoint(loc, around);
    this.map.update();
    if (!combinedResult.noInertia) {
      this.inertia.record(combinedResult);
    }
    this.fireEvents(combinedEventsInProgress, deactivatedHandlers);
  }
  fireEvents(newEventsInProgress, deactivatedHandlers) {
    const wasMoving = isMoving(this.eventsInProgress);
    const nowMoving = isMoving(newEventsInProgress);
    const startEvents = {};
    for (const eventName in newEventsInProgress) {
      if (newEventsInProgress[eventName]) {
        const { originalEvent } = newEventsInProgress[eventName];
        if (!this.eventsInProgress[eventName]) {
          startEvents[`${eventName}start`] = originalEvent;
        }
        this.eventsInProgress[eventName] = newEventsInProgress[eventName];
      }
    }
    if (!wasMoving && nowMoving) {
      this.fireEvent("movestart", nowMoving.originalEvent);
    }
    for (const name in startEvents) {
      if (startEvents[name]) {
        this.fireEvent(name, startEvents[name]);
      }
    }
    if (newEventsInProgress.rotate) {
      this.bearingChanged = true;
    }
    if (nowMoving) {
      this.fireEvent("move", nowMoving.originalEvent);
    }
    for (const eventName in newEventsInProgress) {
      if (newEventsInProgress[eventName]) {
        const { originalEvent } = newEventsInProgress[eventName];
        this.fireEvent(eventName, originalEvent);
      }
    }
    const endEvents = {};
    let originalEndEvent;
    for (const eventName in this.eventsInProgress) {
      if (this.eventsInProgress[eventName]) {
        const { handlerName, originalEvent } = this.eventsInProgress[eventName];
        if (!this.handlersById[handlerName].isActive()) {
          delete this.eventsInProgress[eventName];
          originalEndEvent = deactivatedHandlers[handlerName] || originalEvent;
          endEvents[`${eventName}end`] = originalEndEvent;
        }
      }
    }
    for (const name in endEvents) {
      if (endEvents[name]) {
        this.fireEvent(name, endEvents[name]);
      }
    }
    const stillMoving = isMoving(this.eventsInProgress);
    if ((wasMoving || nowMoving) && !stillMoving) {
      this.updatingCamera = true;
      const inertialEase = this.inertia.onMoveEnd(this.map.dragPan.inertiaOptions);
      const shouldSnapToNorth = (bearing) => bearing !== 0 && -this.bearingSnap < bearing && bearing < this.bearingSnap;
      if (inertialEase) {
        if (shouldSnapToNorth(inertialEase.bearing || this.map.getBearing())) {
          inertialEase.bearing = 0;
        }
        this.map.easeTo(inertialEase, { originalEvent: originalEndEvent });
      } else {
        this.map.emit("moveend", new import_event.Event("moveend", { originalEvent: originalEndEvent }));
        if (shouldSnapToNorth(this.map.getBearing())) {
          this.map.resetNorth();
        }
      }
      this.bearingChanged = false;
      this.updatingCamera = false;
    }
  }
  fireEvent(type, e) {
    this.map.emit(type, new import_event.Event(type, e ? { originalEvent: e } : {}));
  }
};
var handler_manager_default = HandlerManager;
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {});
