var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target, mod));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/handler/scroll_zoom.ts
var scroll_zoom_exports = {};
__export(scroll_zoom_exports, {
  default: () => scroll_zoom_default
});
module.exports = __toCommonJS(scroll_zoom_exports);
var import_lng_lat = __toESM(require("../geo/lng_lat"));
var import_util = require("../util");
var import_dom = __toESM(require("../utils/dom"));
var wheelZoomDelta = 4.000244140625;
var defaultZoomRate = 1 / 100;
var wheelZoomRate = 1 / 450;
var maxScalePerFrame = 2;
var ScrollZoomHandler = class {
  constructor(map, handler) {
    this.onScrollFrame = () => {
      if (!this.frameId) {
        return;
      }
      this.frameId = null;
      if (!this.isActive()) {
        return;
      }
      const tr = this.map.transform;
      if (this.delta !== 0) {
        const zoomRate = this.type === "wheel" && Math.abs(this.delta) > wheelZoomDelta ? this.wheelZoomRate : this.defaultZoomRate;
        let scale = maxScalePerFrame / (1 + Math.exp(-Math.abs(this.delta * zoomRate)));
        if (this.delta < 0 && scale !== 0) {
          scale = 1 / scale;
        }
        const fromScale = typeof this.targetZoom === "number" ? tr.zoomScale(this.targetZoom) : tr.scale;
        this.targetZoom = Math.min(tr.maxZoom, Math.max(tr.minZoom, tr.scaleZoom(fromScale * scale)));
        if (this.type === "wheel") {
          this.startZoom = tr.zoom;
          this.easing = this.smoothOutEasing(200);
        }
        this.delta = 0;
      }
      const targetZoom = typeof this.targetZoom === "number" ? this.targetZoom : tr.zoom;
      const startZoom = this.startZoom;
      const easing = this.easing;
      let finished = false;
      let zoom;
      if (this.type === "wheel" && startZoom && easing) {
        const t = Math.min(((0, import_util.now)() - this.lastWheelEventTime) / 200, 1);
        const k = easing(t);
        zoom = (0, import_util.interpolate)(startZoom, targetZoom, k);
        if (t < 1) {
          if (!this.frameId) {
            this.frameId = true;
          }
        } else {
          finished = true;
        }
      } else {
        zoom = targetZoom;
        finished = true;
      }
      this.active = true;
      if (finished) {
        this.active = false;
        this.finishTimeout = setTimeout(() => {
          this.zooming = false;
          this.handler.triggerRenderFrame();
          delete this.targetZoom;
          delete this.finishTimeout;
        }, 200);
      }
      return {
        noInertia: true,
        needsRenderFrame: !finished,
        zoomDelta: zoom - tr.zoom,
        around: this.aroundPoint,
        originalEvent: this.lastWheelEvent
      };
    };
    this.map = map;
    this.el = map.getCanvasContainer();
    this.handler = handler;
    this.delta = 0;
    this.defaultZoomRate = defaultZoomRate;
    this.wheelZoomRate = wheelZoomRate;
  }
  setZoomRate(zoomRate) {
    this.defaultZoomRate = zoomRate;
  }
  setWheelZoomRate(zoomRate) {
    this.wheelZoomRate = zoomRate;
  }
  isEnabled() {
    return !!this.enabled;
  }
  isActive() {
    return !!this.active || this.finishTimeout !== void 0;
  }
  isZooming() {
    return !!this.zooming;
  }
  enable(options) {
    if (this.isEnabled()) {
      return;
    }
    this.enabled = true;
    this.aroundCenter = options && options.around === "center";
  }
  disable() {
    if (!this.isEnabled()) {
      return;
    }
    this.enabled = false;
  }
  wheel(e) {
    if (!this.isEnabled()) {
      return;
    }
    let value = e.deltaMode === window.WheelEvent.DOM_DELTA_LINE ? e.deltaY * 40 : e.deltaY;
    const nowTime = (0, import_util.now)();
    const timeDelta = nowTime - (this.lastWheelEventTime || 0);
    this.lastWheelEventTime = nowTime;
    if (value !== 0 && value % wheelZoomDelta === 0) {
      this.type = "wheel";
    } else if (value !== 0 && Math.abs(value) < 4) {
      this.type = "trackpad";
    } else if (timeDelta > 400) {
      this.type = null;
      this.lastValue = value;
      this.timeout = setTimeout(this.onTimeout, 40, e);
    } else if (!this.type) {
      this.type = Math.abs(timeDelta * value) < 200 ? "trackpad" : "wheel";
      if (this.timeout) {
        clearTimeout(this.timeout);
        this.timeout = null;
        value += this.lastValue;
      }
    }
    if (e.shiftKey && value) {
      value = value / 4;
    }
    if (this.type) {
      this.lastWheelEvent = e;
      this.delta -= value;
      if (!this.active) {
        this.start(e);
      }
    }
    e.preventDefault();
  }
  renderFrame() {
    return this.onScrollFrame();
  }
  reset() {
    this.active = false;
  }
  onTimeout(initialEvent) {
    this.type = "wheel";
    this.delta -= this.lastValue;
    if (!this.active && this.start) {
      this.start(initialEvent);
    }
  }
  start(e) {
    if (!this.delta) {
      return;
    }
    if (this.frameId) {
      this.frameId = null;
    }
    this.active = true;
    if (!this.isZooming()) {
      this.zooming = true;
    }
    if (this.finishTimeout) {
      clearTimeout(this.finishTimeout);
      delete this.finishTimeout;
    }
    const pos = import_dom.default.mousePos(this.el, e);
    this.around = import_lng_lat.default.convert(this.aroundCenter ? this.map.getCenter() : this.map.unproject(pos));
    this.aroundPoint = this.map.transform.locationPoint(this.around);
    if (!this.frameId) {
      this.frameId = true;
      this.handler.triggerRenderFrame();
    }
  }
  smoothOutEasing(duration) {
    let easing = import_util.ease;
    if (this.prevEase) {
      const preEase = this.prevEase;
      const t = ((0, import_util.now)() - preEase.start) / preEase.duration;
      const speed = preEase.easing(t + 0.01) - preEase.easing(t);
      const x = 0.27 / Math.sqrt(speed * speed + 1e-4) * 0.01;
      const y = Math.sqrt(0.27 * 0.27 - x * x);
      easing = (0, import_util.bezier)(x, y, 0.25, 1);
    }
    this.prevEase = {
      start: (0, import_util.now)(),
      duration,
      easing
    };
    return easing;
  }
};
var scroll_zoom_default = ScrollZoomHandler;
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {});
